function mse_pre  = mse_cv(XL,XR,r1_hat,r2_hat,D,T1,T2,T13,T23)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Compute MSE for cross valication, given the selected model and the cut of data
% INPUT: 
%  Model Specification: r1_hat, r2_hat, D
%  T1:  pre-break subsample
%  T12: pre-break training sample
%  T13: pre break predictive sample
%  T2:  post break subsample
%  T22: post break training sample
%  T23: post break predictive sample
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                N = size(XR,1);    T=size(XR,2);                           %XL and XR have the same number of perids 

if r1_hat == r2_hat && D==0;                                               % no structural change
%--------------------------------------------------------------------------   
%  step1. get factors from first panel use the whole sample
%--------------------------------------------------------------------------   
    [F_hat,~]   = pc_full(XL,T,r1_hat);                                    % factor estimation using the N-traing sample
    F_hat_reg   = F_hat(T13+1:end-T23,1:r1_hat);                           % factors for training, T13 before and T23 after are used for validation
    F_hat_pre1  = F_hat(1:T13,1:r1_hat);                                   % factors for pre-break validation
    F_hat_pre2  = F_hat(end-T23+1:end,1:r1_hat);                           % factors for post-break validation
%--------------------------------------------------------------------------   
%  step2. regress second panel on the generated factors from first panel    
%--------------------------------------------------------------------------   
    XR_Reg      = XR(:,T13+1:end-T23);
    loading_reg = (F_hat_reg'*F_hat_reg)\F_hat_reg'*XR_Reg';               % inv(A)*b=A\b, rxN matrix
%--------------------------------------------------------------------------   
%  step 3. use the lower corner of the right sample for prediction  
%--------------------------------------------------------------------------  
    XR_Pre1     = XR(:,1:T13);
    XR_Pre2     = XR(:,end-T23+1:end); 
    e_pre1      = XR_Pre1 - loading_reg'*F_hat_pre1';                      % N by T13
    e_pre2      = XR_Pre2 - loading_reg'*F_hat_pre2';                      % N by T23
    mse_pre     = norm(e_pre1,'fro')^2/N/T13+norm(e_pre2,'fro')^2/N/T23;   % avg predicted MSE for all series
else         
%--------------------------------------------------------------------------                                                                 
                  % First, Post-Break Subsample
%--------------------------------------------------------------------------   
%  step1. get factors from left panel use the post-break subsample
%--------------------------------------------------------------------------    
    XL_Reg2      = XL(:,end-T2+1:end);                                     % only use the post-break subsample for factor estiamtion
    [F_hat,~]    = pc_full(XL_Reg2,T2,r2_hat);
    F_hat_reg2   = F_hat(1:end-T23,1:r2_hat);                              % use for regression
    F_hat_pre2   = F_hat(end-T23+1:end,1:r2_hat);                          % use for predictor, T23 periods
%--------------------------------------------------------------------------   
%  step2. regress right panel on the generated factors from left panel    
%--------------------------------------------------------------------------    
    XR_Reg2      = XR(:,end-T2+1:end-T23);
    loading_reg2 = (F_hat_reg2'*F_hat_reg2)\F_hat_reg2'*XR_Reg2';          % inv(A)*b=A\b  
%--------------------------------------------------------------------------   
%  step 3. use the lower corner of the right sample for prediction
%--------------------------------------------------------------------------   
    XR_Pre2      = XR(:,end-T23+1:end);   
    e_pre2       = XR_Pre2 - loading_reg2'*F_hat_pre2';
%--------------------------------------------------------------------------      
                 % Second, Pre-Break Subsample
%--------------------------------------------------------------------------   
%  step1. get factors from left panel use the post-break subsample
%--------------------------------------------------------------------------    
    XL_Reg1      = XL(:,1:T1);                                             % only use the pre-break subsample for factor estiamtion
    [F_hat,~]    = pc_full(XL_Reg1,T1,r1_hat);
    F_hat_reg1   = F_hat(T13+1:T1,1:r1_hat);                               % use for regression
    F_hat_pre1   = F_hat(1:T13,1:r1_hat);                                  % use for predictor, T3 periods
%--------------------------------------------------------------------------   
%  step2. regress right panel on the generated factors from left panel   
%--------------------------------------------------------------------------   
    XR_Reg1      = XR(:,(T13+1):T1);
    loading_reg1 = (F_hat_reg1'*F_hat_reg1)\F_hat_reg1'*XR_Reg1';          % inv(A)*b=A\b    
%--------------------------------------------------------------------------   
%  step 3. use the lower corner of the right sample for prediction   
%--------------------------------------------------------------------------   
    XR_Pre1      = XR(:,1:T13);
    e_pre1       = XR_Pre1 - loading_reg1'*F_hat_pre1';
    mse_pre      = norm(e_pre1,'fro')^2/N/T13+norm(e_pre2,'fro')^2/N/T23;  % avg predicted MSE for all series
   
end

end
    
    

