%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This programs estimate the structure of the Model, factors, and loadings
% INPUT: 
%  X:  whole sample
%  tao: break point
%  Tuning: two penalization tuning parameter
%  k: max of number of factors in each subsample
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [r1_hat,r2_hat,D] = selectedmodel(X,tao,cons1,cons2,k,options)

         [N,T]  = size(X); T1 = floor(T*tao); T2 = T-T1;
%--------------------------------------------------------------------------
%               define the groups for group penalization
%--------------------------------------------------------------------------
groups = zeros(2*N*k,1);                                                   
for  i = 1:k   
     s = (i-1)*N+1;     e = i*N;     groups(s:e) = i;    
     s = (k+(i-1))*N+1; e =(k+i)*N;  groups(s:e) = k+i;
end
%--------------------------------------------------------------------------
%                  Construct the tuning parameter 
%--------------------------------------------------------------------------
    [Tuning, ~] = tuningpara12(X,tao,k,cons1,cons2,options);
%--------------------------------------------------------------------------
%      principle component estimator of factors and loadings
%--------------------------------------------------------------------------
      [F1_til,F2_til,lambda1_til,lambda2_til] = pc(X,T1,T2,k); 
               gamma_til = lambda2_til-lambda1_til; 
%--------------------------------------------------------------------------
%                       Shrinkage estimation
%--------------------------------------------------------------------------  
        funObj  = @(para)pel_ls(para,X,F1_til,F2_til);          
        fpara   = [lambda1_til gamma_til];
        fpara0  = reshape(fpara,2*N*k,1); 
        paraest = L1GeneralGroup_Auxiliary(funObj,fpara0,Tuning,groups,options);
        fest    = reshape(paraest,N,2*k);
%--------------------------------------------------------------------------
%          Find the model recovered by shrinkage estimation
%--------------------------------------------------------------------------  
        A = diag(fest'*fest); A1 = A(1:k); A2 = A(k+1:2*k);
 
        if norm(A1) == 0
           r1_hat    = 0;
        else
           r1_hat    = find(A1,1,'last'); 
        end

        % post-breat number of factors
        if norm(A2) == 0
           r2_hat    = r1_hat;
        else
           r2_hat    = find(A2,1,'last');
           r2_hat    = max(r1_hat,r2_hat);
        end
        
        gamma = fest(:,k+1:2*k);
        D     = norm(gamma(:,1:r1_hat),'fro');
        
        
        