function [r1_hat,r2_hat,D] = selectedmodel_unknown(X,taov,cons1,cons2,k,options)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This programs estimate the structure of the Model, factors, and loadings
% when the break date is unknown
% INPUT: 
%  X:  whole sample
%  taov: break point interval
%  Tuning: two penalization tuning parameter
%  k: max of number of factors in each subsample
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

[N,T]  =    size(X);    ntao    =   size(taov,2);

%--------------------------------------------------------------------------
%               allocate memory for saving matrix
%--------------------------------------------------------------------------

    rv = zeros(2,ntao);  paraestv=zeros(2*N*k,ntao);  Dv = zeros(1,ntao);

%--------------------------------------------------------------------------
%               define the groups for group penalization
%--------------------------------------------------------------------------
groups = zeros(2*N*k,1);                                                   
for  i = 1:k   
     s = (i-1)*N+1;     e = i*N;     groups(s:e) = i;    
     s = (k+(i-1))*N+1; e =(k+i)*N;  groups(s:e) = k+i;
end

%--------------------------------------------------------------------------
%                  Construct the tuning parameter 
%--------------------------------------------------------------------------
    [Tuning, fpara0] = tuningpara12_unknown(X,taov,k,cons1,cons2,options);       % select the same tuning and starting for all break dates
%--------------------------------------------------------------------------
%                  loop over break dates for model sel
%-------------------------------------------------------------------------

for j = 1:ntao
    
    tao = taov(j);
    T1  = floor(T*tao);     T2  =    T-T1; 
    
%--------------------------------------------------------------------------
%          principle component estimator of factors and loadings 
%--------------------------------------------------------------------------

    [F1_til,F2_til,~,~] = pc(X,T1,T2,k);                                   %principle component estimator of factors and loadings
    
%--------------------------------------------------------------------------
%           Shrinkage Estimation for Each Break Date
%--------------------------------------------------------------------------
   
    funObj   = @(para)pel_ls(para,X,F1_til,F2_til);                        %criterion function function
    paraest0 = L1GeneralGroup_Auxiliary(funObj,fpara0,Tuning,groups,options);
    fest0    = reshape(paraest0,N,2*k);                                    %updated shrinkage estimator
    
%--------------------------------------------------------------------------
%          Model Selection for Each Break Date
%--------------------------------------------------------------------------
    A = diag(fest0'*fest0); A1 = A(1:k); A2 = A(k+1:2*k);
        
    if norm(A1) == 0; 
        er1 = 0;   
    else
        er1 = find(A1,1,'last'); 
    end

    if norm(A2) == 0; 
        er2 = er1; 
    else
        er2 = find(A2,1,'last'); er2 = max(er1,er2); 
    end
     

        
    gamma = fest0(:,k+1:2*k);
    D     = norm(gamma(:,1:er1),'fro');
    

%--------------------------------------------------------------------------
%          Save Results for Each Break Date
%--------------------------------------------------------------------------
 rv(:,j)        =   [er1;er2];
 paraestv(:,j)  =   paraest0;
 Dv(:,j)        =   D;
 
end

%--------------------------------------------------------------------------
%          Final  Model Selection Result
%--------------------------------------------------------------------------
r1_hat_V = rv(1,:);   r1_hat   = min(r1_hat_V);
r2_hat_V = rv(2,:);   r2_hat   = min(r2_hat_V);

D=norm(Dv); % end of model selection results, r1_hat, r2_hat, D

    

    
    
        
        
        