function [Tuning, fpara0] = tuningpara12_unknown(X,taov,k,cons1,cons2,options)
% This program generalize tuningpara12.m to the unknown break date case by
% replacing tao with a vector taov. Average tuning is applied.
% Given constant cons1 and cons2, this program gives the averaging penalty
% Tuning and the starting value fpara0


%%------------------------------------------------------------------------
                        dp = 2;   [N,T]  = size(X);             
%--------------------------------------------------------------------------
    groups = zeros(2*N*k,1); 
for i = 1:k   
    s = (i-1)*N+1;     e = i*N;   
    groups(s:e) = i;
    s = (k+(i-1))*N+1; e = (k+i)*N; 
    groups(s:e) = k+i;  
end  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                    Step 1. first step tuning parameter                  %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                      
%                       locate memory for saving matrix
%%-------------------------------------------------------------------------
ntao     =  size(taov,2);  
TuningV  =  zeros(2*k,ntao);     fpara0V  =  zeros(2*N*k,ntao); 
normlV   =  zeros(k,ntao);       normgV   =  zeros(k,ntao);

%%-------------------------------------------------------------------------
%                       loop over different break dates
%%-------------------------------------------------------------------------
for j = 1:ntao
    
    tao = taov(j);
    T1  = floor(T*tao);     T2  =    T-T1; 
    X1  = X(:,1:T1);        X2  =    X(:,T1+1:end);
%--------------------------------------------------------------------------
%          principle component estimator of factors and loadings 
%--------------------------------------------------------------------------
        [F1_til,F2_til,lambda1_til,lambda2_til] = pc(X,T1,T2,k);  
%--------------------------------------------------------------------------
%              preliminary estimator of residuals
%--------------------------------------------------------------------------    
        R1 = X1-lambda1_til*F1_til';  R2 = X2-lambda2_til*F2_til';  
%--------------------------------------------------------------------------
%                preliminary tuning parameter
%--------------------------------------------------------------------------   
        e1 = norm(R1,'fro')/(N*T1)^.5+norm(R2,'fro')/(N*T2)^.5;
        e2 = norm(R2,'fro')/(N*T2)^.5;

        Ftuning10 = cons1*N^(-.5)*max((T1)^(-(dp+1)/2),N^(-(dp+1)/2));     % a constant in the tuning parameter for penalization
        Ftuning20 = cons2*N^(-.5)*max((T2)^(-(dp+1)/2),N^(-(dp+1)/2));     % a constant in the tuning parameter for penalization

        norml     = sqrt(diag(lambda1_til'*lambda1_til))';                 % column norms of Lambda1, row vector
        gamma_til = lambda2_til-lambda1_til; 
        normg     = sqrt(diag(gamma_til'*gamma_til))';                     % column norms of Gamma, row vector
        
        norml2    = sqrt(diag(lambda2_til'*lambda2_til))';                 % column norms of Lambda2
        normg     = min([normg;norml2]);                                   % adjustment needed for unknown break date case
        
        Ftuning1  = e1*Ftuning10*N^dp*(norml.^(-2*dp));
        Ftuning2  = e2*Ftuning20*N^dp*(normg.^(-2*dp));
        
        fpara     = [lambda1_til gamma_til];
        fpara0    = reshape(fpara,2*N*k,1);                                % starting values
        Tuning    = [Ftuning1';Ftuning2'];                                 % column vector
        
        TuningV(:,j)  =  Tuning; 
        fpara0V(:,j)  =  fpara0;
        normlV(:,j)   =  norml;
        normgV(:,j)   =  normg;
end

%--------------------------------------------------------------------------
%                   average tuning parameter across break dates
%--------------------------------------------------------------------------
         
        Tuning   =   mean(TuningV,2);  
        fpara0   =   mean(fpara0V,2); 

         
        
        
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Step 2. Shrinkage Est and Model Sel Based on first step Tuning           %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%--------------------------------------------------------------------------
%                       locate memory for saving matrix
%--------------------------------------------------------------------------
rv  =  zeros(2,ntao); 
%--------------------------------------------------------------------------

for j = 1:ntao
    
    tao = taov(j);
    T1  = floor(T*tao); T2=T-T1;
        
%--------------------------------------------------------------------------
%          principle component estimator of factors and loadings 
%--------------------------------------------------------------------------

    [F1_til,F2_til,~,~] = pc(X,T1,T2,k);                                   %principle component estimator of factors and loadings
    
%--------------------------------------------------------------------------
%           Shrinkage Estimation for Each Break Date
%--------------------------------------------------------------------------
   
    funObj   = @(para)pel_ls(para,X,F1_til,F2_til);                        %criterion function function
    paraest0 = L1GeneralGroup_Auxiliary(funObj,fpara0,Tuning,groups,options);
    fest0    = reshape(paraest0,N,2*k);                                    %updated shrinkage estimator
    
%--------------------------------------------------------------------------
%          Model Selection for Each Break Date
%--------------------------------------------------------------------------
    A = diag(fest0'*fest0); A1 = A(1:k); A2 = A(k+1:2*k);
        
    if norm(A1) == 0; 
        er1 = 0;   
    else
        er1 = find(A1,1,'last'); 
    end

    if norm(A2) == 0; 
        er2 = er1; 
    else
        er2 = find(A2,1,'last'); er2 = max(er1,er2); 
    end
    
    
%--------------------------------------------------------------------------
%          Save Results for Each Break Date
%--------------------------------------------------------------------------   
    rv(:,j) = [er1;er2];
%--------------------------------------------------------------------------
	
end                                                                        %end of loop over break dates

%--------------------------------------------------------------------------
%      model selected by  initial tuning parameter
%--------------------------------------------------------------------------   
      
            r1_hat_V = rv(1,:);     er1   = min(r1_hat_V);
            r2_hat_V = rv(2,:);     er2   = min(r2_hat_V);
 
 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                   Step 3. Second Step Tuning Parameter                  %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
for j = 1:ntao
    
    tao = taov(j);
    T1  = floor(T*tao);     T2  =    T-T1; 
    X1  = X(:,1:T1);        X2  =    X(:,T1+1:end);
    

    [F1_til,F2_til,lambda1_til,lambda2_til] = pc(X,T1,T2,k);             % principle component estimator of factors and loadings
    
    Ftuning10 = cons1*N^(-.5)*max((T1)^(-(dp+1)/2),N^(-(dp+1)/2));         % a constant in the tuning parameter for penalization
    Ftuning20 = cons2*N^(-.5)*max((T2)^(-(dp+1)/2),N^(-(dp+1)/2));         % a constant in the tuning parameter for penalization


%--------------------------------------------------------------------------
%        preliminary weights component saved from first step
%--------------------------------------------------------------------------
        norml  =  normlV(:,j)';     normg  =   normgV(:,j)';
%--------------------------------------------------------------------------
%       update using first step selection results
%--------------------------------------------------------------------------
        F1_se      = F1_til(:,1:er1);      
        lambda1_se = lambda1_til(:,1:er1);
        F2_se      = F2_til(:,1:er2);      
        lambda2_se = lambda2_til(:,1:er2);
        R1_se      = X1-lambda1_se*F1_se'; 
        R2_se      = X2-lambda2_se*F2_se';
        e1         = norm(R1_se,'fro')/(N*T1)^.5+norm(R2_se,'fro')/(N*T2)^.5;
        e2         = norm(R2_se,'fro')/(N*T2)^.5; 

    if er1==er2
        lambda2_tr = rotation_lamb2(lambda1_se,lambda2_se,er1); 
    else
        lambda2_tr = lambda2_se;
    end
 
         gamma_se       = lambda2_tr-[lambda1_se,zeros(N,er2-er1)]; 
         normg_se       = sqrt(diag(gamma_se'*gamma_se))';                 % column norms of Gamma_se
         normg_n        = normg;
         
         norml2_se      = sqrt(diag(lambda2_tr'*lambda2_tr))';
         normg_se       = min([normg_se;norml2_se]);
         
         
         normg_n(1:er2) = normg_se;                                        % update norm
            
         Ftuning1       = e1*Ftuning10*N^dp*(norml.^(-2*dp));
         Ftuning2       = e2*Ftuning20*N^dp*(normg_n.^(-2*dp));
         Tuning         = [Ftuning1';Ftuning2'];  
         
         TuningV(:,j)   = Tuning;  
        
end

%--------------------------------------------------------------------------
%                   average tuning parameter across break dates
%--------------------------------------------------------------------------
         
                 Tuning      =   mean(TuningV,2);


                
               
        
          

 
   
    
    
    
    
