% Based on the paper by X.Cheng, Z.Liao and F.Schorfheide 
% Shrinkage Estimation of High-Dimensional Factor
% Models with Structural Instabilities
% This Version: 07/09/2015
%--------------------------------------------------------------------------
%                       0. Housekeeping
%-------------------------------------------------------------------------- 
clear; clc;

options = optimset('MaxFunEvals', 200000, 'MaxIter', 500);
path    = pwd;
addpath([pwd, '/CLSFactorProg']);
addpath([pwd, '/OptimizationFunctions']);
load('SW_xdata_ver03_long.mat');    


data_order_by_corr                                                         %order series by correlation


X = xdata_std'; [N,T] = size(X);                                           

NRep = 5; TRep = 10; k = 8;                                                %5 fold N-CV, 10 T-rolling windows, largest number of factors allowed is 8


 
groups  = zeros(2*N*k,1);                                                  % define the groups for group penalization
for i   = 1:k   
    s   = (i-1)*N+1; e = i*N;   
    groups(s:e) = i;
    s   = (k+(i-1))*N+1; e = (k+i)*N; 
    groups(s:e) = k+i;  
end

head = {'IntSize','r1','r2-r1','BreakIndicator','LSBreak','ReviseBreak','Cons1','Cons2'};

tic

%-------------------------------------------------------------------------- 
%                       1. Known break date
%-------------------------------------------------------------------------- 
                  T1 = 276; T2 = T - T1; tao_0 = T1/T;                     % NBER dated recession, 2007M12

[F1_til,F2_til,~,~] = pc(X,T1,T2,k); 

[Tuning,fpara0,cons1,cons2] = tuningpara_cv(X,T1,T2,tao_0,k,NRep,TRep,options); 

funObj   = @(para)pel_ls(para,X,F1_til,F2_til);  

paraest = L1GeneralGroup_Auxiliary(funObj,fpara0,Tuning,groups,options);

 fest    = reshape(paraest,N,2*k);

 A = diag(fest'*fest); A1 = A(1:k); A2 = A(k+1:2*k);

        if norm(A1) == 0; 
            r1_hat = 0;      
        else
            r1_hat = find(A1,1,'last');                              
        end
        
        if norm(A2) == 0; 
            r2_hat = r1_hat; 
        else
            r2_hat = find(A2,1,'last'); r2_hat = max(r1_hat,r2_hat); 
        end
        
        gamma = fest(:,k+1:2*k);
        
 D  = norm(gamma(:,1:r1_hat),'fro');
 
 if r1_hat == r2_hat && D==0
     S=0;
 else
     S=1;     
 end


rslt_known = [0,r1_hat,r2_hat-r1_hat,S,T1,T1,cons1,cons2];
rslt_known = num2cell(rslt_known);
 


 
%-------------------------------------------------------------------------- 
%                       1. Unknown break date
%--------------------------------------------------------------------------         


intsize = 3;
    

T1 = 276; tao_0 = T1/T;
step    = 1/T;                 tao_l  = tao_0-step*intsize;     tao_u = tao_0+step*intsize; 
taov    = tao_l:step:tao_u;    ntao   = size(taov,2);                        
T1Early =  floor(T*min(taov)); T1Late = floor(T*max(taov));     
Ta = T1Early;                  Tb     = T-T1Late;                                                   
                                                                
[Tuning,fpara0,cons1,cons2] = tuningpara_cv_unknown(X,Ta,Tb,taov,k,NRep,TRep,options);
%--------------------------------------------------------------------------
%                 load memory for matrix saving
%--------------------------------------------------------------------------        
    rv = zeros(2,ntao); paraestv=zeros(2*N*k,ntao); Dv = zeros(1,ntao);
%--------------------------------------------------------------------------    
for j = 1:ntao
    
            tao = taov(j);
            T1  = floor(T*tao);     T2  =    T-T1; 
%--------------------------------------------------------------------------
%          principle component estimator of factors and loadings 
%--------------------------------------------------------------------------
            [F1_til,F2_til,~,~] = pc(X,T1,T2,k);                           %principle component estimator of factors and loadings  
%--------------------------------------------------------------------------
%           Shrinkage Estimation for Each Break Date
%--------------------------------------------------------------------------
            funObj   = @(para)pel_ls(para,X,F1_til,F2_til);                %criterion function function
            paraest0 = L1GeneralGroup_Auxiliary(funObj,fpara0,Tuning,groups,options);
            fest0    = reshape(paraest0,N,2*k);                                    %updated shrinkage estimator
%--------------------------------------------------------------------------
%          Model Selection for Each Break Date
%--------------------------------------------------------------------------
            A = diag(fest0'*fest0); A1 = A(1:k); A2 = A(k+1:2*k);
        
            if norm(A1) == 0; 
                er1 = 0;   
            else
                er1 = find(A1,1,'last'); 
            end

            if norm(A2) == 0; 
                er2 = er1; 
            else
                er2 = find(A2,1,'last'); er2 = max(er1,er2); 
            end
      
            gamma = fest0(:,k+1:2*k);
            D     = norm(gamma(:,1:er1),'fro');
%--------------------------------------------------------------------------
%          Save Results for Each Break Date
%--------------------------------------------------------------------------
            rv(:,j)        =   [er1;er2];
            paraestv(:,j)  =   paraest0;
            Dv(:,j)        =   D;
 
end
%--------------------------------------------------------------------------
%          Final  Model Selection Result
%--------------------------------------------------------------------------
            r1_hat_V = rv(1,:);   r1_hat   = min(r1_hat_V) ;
            r2_hat_V = rv(2,:);   r2_hat   = min(r2_hat_V) ;
            D        = norm(Dv) ;
            
            if r1_hat == r2_hat && D==0
                S=0;
            else
                S=1;
            end

%--------------------------------------------------------------------------   
%             Break point Estimation
%--------------------------------------------------------------------------
          numPeriods =  length(T1Early:T1Late);                            % number of possible break dates
           residsHat =  zeros(numPeriods,1);
%--------------------------------------------------------------------------
for j = 1:ntao
        
     tao = taov(j);     
     T1  = floor(T*tao);                T2 = T- T1;        
     XBefore =  X(:,1:T1);              XAfter = X(:,T1+1:T);              %split sample
        
%--------------------------------------------------------------------------
%      principle component estimation of two subsample
%--------------------------------------------------------------------------
[F1_til,F2_til,lambda1_til,lambda2_til] = pc(X,T1,T2,k);  

%--------------------------------------------------------------------------
%      impose the selected number of factors
%--------------------------------------------------------------------------
  
     R1 = XBefore - lambda1_til(:,1:r1_hat)*F1_til(:,1:r1_hat)';         
     R2 = XAfter - lambda2_til(:,1:r2_hat)*F2_til(:,1:r2_hat)';        
     residsHat(j) = (norm(R1,'fro')^2+norm(R2,'fro')^2)/(N*T);
     
end

jr         = find(residsHat==min(residsHat));
tao_hat    = taov(jr);
T1_hat     = T1Early+jr-1;

    
%--------------------------------------------------------------------------
%       Revised break point estimation
%-------------------------------------------------------------------------
% 
% - Choose the points closest to the conjecture point (among parsimous models)

rsum    = rv(1,:)+rv(2,:);
rset    = find(rsum==min(rsum));
rmid    = ceil(size(rv,2)+1)/2; %this assumes the conjecture point is in the middle of the searching interval
rbreak  = find(taov == tao_0); % exact conjectured break point
rdist   = abs(rset-rbreak);
[a1,a2] = min(rdist);
jr_rev     = rset(a2);
T1_hat_rev = T1Early+jr_rev-1;


%--------------------------------------------------------------------------
 
rslt_unknown = [intsize,r1_hat,r2_hat-r1_hat,S,T1_hat,T1_hat_rev,cons1,cons2];

rslt_unknown = num2cell(rslt_unknown);
 

%--------------------------------------------------------------------------


rslt = [head; rslt_known; rslt_unknown];


save('Empirical_CLS.mat','rslt');





